// transfer file over TCP/IP
// created by Zachary Seldess, 10/26/12
// Copyright  2012. All Rights Reserved.

import java.io.*;
import java.net.*;
import com.cycling74.max.*;

public class TCPFileSender extends MaxObject {

    private String host;
    private int port;

    public TCPFileSender() {
        declareIO(1,2);
        setInletAssist(0, "transfer file, set destination host and port");
        setOutletAssist(1, "1 for successfull file load, 0 for failure");
        setOutletAssist(0, "1 for successfull transfer, 0 for failure");
        createInfoOutlet(false);

        host = "localhost";
        port = 7474;
        declareAttribute("host");
        declareAttribute("port");

        //post("TCPFileSender by Zachary Seldess. Build October 26 2012.");
        //post("Copyright  2012. All Rights Reserved.");
    }

    public void send(Atom[] args) {
        File fileToServer = null;
        boolean fileReadSuccessful = true;
        boolean fileTransferSuccessful = true;

        try {
            fileToServer = new File(args[0].toString());
        }
        catch (Exception e) {
            fileReadSuccessful = false;
            //post("ERROR: File could not be read. Transfer terminated. " + e.getMessage() + ".");
        }

        if (fileReadSuccessful) {
            Socket clientSocket = null;

            try {
                clientSocket = new Socket(host, port);
                //post(clientSocket.isConnected());
            }
            catch (Exception e) {
                fileTransferSuccessful = false;
                //post("ERROR: Socket could not make connection. " + e.getMessage() + ".");
            }
            
            FileInputStream fis = null;

            if (clientSocket != null) {
                try {
                    DataOutputStream dos = new DataOutputStream(clientSocket.getOutputStream());
                    
                    // send file name in first byte array
                    byte[] fileNameBytes = fileToServer.getName().getBytes();
                    // if name >= 1024 bytes, truncate
                    if (fileNameBytes.length >= 1024) {
                        dos.write(fileNameBytes, 0, 1024);
                    }
                    // otherwise pad byte array as necessary
                    else {
                        byte[] initialChunk = new byte[1024];
                        System.arraycopy(fileNameBytes, 0, initialChunk, 0, fileNameBytes.length);
                        dos.write(initialChunk, 0, 1024);
                    }
                    //post("Sending File: " + fileToServer.getName());

                    fis = new FileInputStream(fileToServer);

                    // then send contents of file in future byte arrays
                    byte[] b = new byte[1024];
                    int count;
                    while ((count = fis.read(b)) != -1) {
                        dos.write(b, 0, count);
                    }
                }
                catch (IOException e) {
                    fileTransferSuccessful = false;
                    //post("ERROR: Unsuccessful file transfer. " + e.getMessage() + ".");
                }

                try {
                    clientSocket.close();
                    fis.close();
                }
                catch (IOException e) {
                }
            }
        }

        outlet(1, fileReadSuccessful ? 1 : 0);
        outlet(0, fileTransferSuccessful ? 1 : 0);
    }

    public void inlet(int i) {
        // do nothing
    }

    public void inlet(float f) {
        // do nothing
    }

    public void list(Atom[] args) {
        // do nothing
    }

    public void anything(String msg, Atom[] args) {
        // do nothing
    }

    public void bang() {
        // do nothing
    }

}




